package authhelper

import (
	"errors"
	"fmt"
	"strings"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/coreospackage/logginghelper"
	jwt "github.com/dgrijalva/jwt-go"
)

// // JwtCustomClaims are custom claims extending default ones.
// type JwtCustomClaims struct {
// 	Username string `json:"username"`
// 	jwt.StandardClaims
// }

// // GetJwtConfig middleware with the custom claims type
// func GetJwtConfig() middleware.JWTConfig {
// 	jwtConfig := middleware.JWTConfig{
// 		Claims:     &JwtCustomClaims{},
// 		SigningKey: []byte(JwtKey),
// 	}
// 	return jwtConfig
// }

// DecodeToken decode token
func DecodeToken(tokenFromRequest, jwtKey string) (jwt.MapClaims, error) {
	tokenArray := strings.Split(tokenFromRequest, "Bearer")
	if len(tokenArray) <= 1 {
		return nil, errors.New("Provided JWT token is nil or invalid ")
	}
	tokenFromRequest = strings.Trim(tokenArray[1], " ")
	// get data i.e.Claims from token
	token, err := jwt.Parse(tokenFromRequest, func(token *jwt.Token) (interface{}, error) {
		// Don't forget to validate the alg is what you expect:
		_, ok := token.Method.(*jwt.SigningMethodHMAC)
		if !ok {
			return nil, fmt.Errorf("Unexpected signing method: %v", token.Header["alg"])
		}
		return []byte(jwtKey), nil
	})

	if err != nil {
		logginghelper.LogError("Error while parsing JWT Token: ", err)
		return nil, err
	}
	claims, ok := token.Claims.(jwt.MapClaims)
	if !ok {
		return nil, errors.New("Error while getting claims")
	}
	return claims, nil
}
