package cachehelper

import (
	"time"

	"github.com/bluele/gcache"
)

// CacheGCHelper use this to create new cache object
// Remember it holds memory
type CacheGCHelper struct {
	GC         gcache.Cache
	Expiration time.Duration
	MaxEntries int
}

//Setup create new object of GC
func (gchelper *CacheGCHelper) Setup(MaxEntries int, Expiration time.Duration) {
	gchelper.MaxEntries = MaxEntries
	gchelper.Expiration = Expiration
	gchelper.GC = gcache.New(MaxEntries).
		LFU().
		Build()
}

//Setup create new object of GC
func (gchelper *CacheGCHelper) SetupWithCallbackFunction(MaxEntries int, Expiration time.Duration, f func(key interface{}, value interface{}) error) {
	gchelper.MaxEntries = MaxEntries
	gchelper.Expiration = Expiration
	gchelper.GC = gcache.New(MaxEntries).
		LFU().
		EvictedFunc(func(key, value interface{}) {
			f(key, value)
		}).
		Build()
}

//New Add new Key and value
func (gchelper *CacheGCHelper) Set(key string, object interface{}) error {
	return gchelper.GC.SetWithExpire(key, object, gchelper.Expiration)
}

//Get object based on key
func (gchelper *CacheGCHelper) Get(key string) (interface{}, error) {
	return gchelper.GC.Get(key)
}

// GetAll objects from gc
func (gchelper *CacheGCHelper) GetAll() map[interface{}]interface{} {
	return gchelper.GC.GetALL()
}

// Remove object from GC
func (gchelper *CacheGCHelper) Remove(key string) bool {
	return gchelper.GC.Remove(key)
}

//Purge all objects
func (gchelper *CacheGCHelper) Purge() {
	gchelper.GC.Purge()
}
