package diskdetailshelper

import (
	"bytes"
	"errors"
	"os/exec"
	"regexp"
	"runtime"
	"strconv"
	"strings"
	"time"

	"github.com/shirou/gopsutil/host"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/coreospackage/diskdetailshelper/gopsutil"
	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/coreospackage/logginghelper"

	"github.com/shirou/gopsutil/cpu"
	"github.com/shirou/gopsutil/mem"
)

// UsageStat struct contains serial number and disk stats
type UsageStat struct {
	SerialNumber string      `json:"serialNumber"`
	DiskStats    []DiskStats `json:"diskStats"`
}

// DiskStats struct contains path, total space, free space, used space and percentage of usage
type DiskStats struct {
	Path        string  `json:"path"`
	Total       uint64  `json:"total"`
	Free        uint64  `json:"free"`
	Used        uint64  `json:"used"`
	UsedPercent float64 `json:"usedPercent"`
}

// Analyse disk analyses the all disks and returns serial number of disk and path, total space, free space, used space, percentage of each drive
// func AnalyseDisk() (UsageStat, error) {
// 	partitionStats, partitionReadError := gopsutil.Partitions(true)
// 	UsageStatobj := UsageStat{}
// 	if partitionReadError != nil {
// 		logginghelper.LogError("error occured while reading partitions", partitionReadError)
// 		return UsageStatobj, partitionReadError
// 	}
// 	cmd := exec.Command("wmic")
// 	cmd.Stdin = strings.NewReader("diskdrive get serialnumber")
// 	var out bytes.Buffer
// 	cmd.Stdout = &out
// 	commandError := cmd.Run()
// 	if commandError != nil {
// 		logginghelper.LogError("error occured while running wmic command", commandError)
// 		return UsageStatobj, commandError
// 	}
// 	diskDetailList := strings.SplitAfter(out.String(), "\n")

// 	if diskDetailList == nil || len(diskDetailList) <= 0 {
// 		logginghelper.LogError("disk detailslist has no elements to show")
// 		return UsageStatobj, errors.New("disk detailslist has no elements to show")
// 	}
// 	// As serial number is placed at 3rd place in diskDetailsList
// 	UsageStatobj.SerialNumber = diskDetailList[2]
// 	for _, elem := range partitionStats {
// 		diskDataObj := DiskStats{}
// 		dsk, usageError := disk.Usage(elem.Device)
// 		if usageError != nil {
// 			logginghelper.LogError("error occured while analysing usage of ", elem.Device)
// 		} else {
// 			diskDataObj.Path = dsk.Path
// 			diskDataObj.Total = dsk.Total / 1073741824
// 			diskDataObj.Used = dsk.Used / 1073741824
// 			diskDataObj.Free = dsk.Free / 1073741824
// 			diskDataObj.UsedPercent = dsk.UsedPercent
// 			UsageStatobj.DiskStats = append(UsageStatobj.DiskStats, diskDataObj)
// 		}
// 	}
// 	return UsageStatobj, nil
// }

// AnalyseDisk analyses the all disks and returns serial number of disk and path, total space, free space, used space, percentage of each drive
func AnalyseDisk() (UsageStat, error) {
	UsageStatobj := UsageStat{}
	os := ""
	osInfo, err := host.Info()
	if err != nil {
		logginghelper.LogError("error occured while getting OS details", err)
		// return UsageStatobj, err
		os = runtime.GOOS
	} else {
		os = osInfo.OS
	}
	switch os {
	case "windows":
		UsageStatobj.SerialNumber = gopsutil.GetDiskSerialNumber("")
		diskStats, err := GetDiskDetails()
		if err != nil {
			logginghelper.LogError("error occured while getting disk stats", err)
			return UsageStatobj, err
		}
		UsageStatobj.DiskStats = diskStats
	case "linux", "darwin":
		// Add code to get linux, MAC OS disk details
		// UsageStatobj.SerialNumber = gopsutil.GetDiskSerialNumber("/dev/sda")
		// partitionStats, partitionReadError := gopsutil.Partitions(true)
		// if partitionReadError != nil {
		// 	logginghelper.LogError("error occured while reading partitions", partitionReadError)
		// 	return UsageStatobj, partitionReadError
		// }
		// for _, elem := range partitionStats {
		// 	diskDataObj := DiskStats{}
		// 	dsk, usageError := disk.Usage(elem.Device)
		// 	if usageError != nil {
		// 		logginghelper.LogError("error occured while analysing usage of ", elem.Device)
		// 		return UsageStatobj, usageError
		// 	}
		// 	diskDataObj.Path = dsk.Path
		// 	diskDataObj.Total = dsk.Total / 1073741824
		// 	diskDataObj.Used = dsk.Used / 1073741824
		// 	diskDataObj.Free = dsk.Free / 1073741824
		// 	diskDataObj.UsedPercent = dsk.UsedPercent
		// 	UsageStatobj.DiskStats = append(UsageStatobj.DiskStats, diskDataObj)
		// }
	}

	return UsageStatobj, nil
}

// GetDiskDetails returns disk details for windows OS
func GetDiskDetails() ([]DiskStats, error) {
	diskStats := []DiskStats{}
	cmd := exec.Command("wmic")
	cmd.Stdin = strings.NewReader("logicaldisk get caption,drivetype,freespace,size")

	var out bytes.Buffer
	cmd.Stdout = &out
	commandError := cmd.Run()
	if commandError != nil {
		logginghelper.LogError("error occured while running wmic command", commandError)
		return diskStats, commandError
	}

	re := regexp.MustCompile("[A-Z]{1}:[\\s]+[0-9][\\s]+[0-9]+[\\s]+[0-9]+")
	diskDetailList := re.FindAllString(out.String(), -1)

	if diskDetailList == nil || len(diskDetailList) == 0 {
		return diskStats, nil
	}
	for _, line := range diskDetailList {
		diskDetail := SplitString(line)
		// FIXED DRIVE
		if len(diskDetail) == 4 && diskDetail[1] == "3" {
			stat := DiskStats{}
			stat.Path = diskDetail[0]
			freeSpace, err := strconv.ParseInt(diskDetail[2], 10, 64)
			if err == nil {
				stat.Free = uint64(freeSpace) / 1073741824
			} else {
				logginghelper.LogError("GetDiskDetails() Error occured while Free space conversion: ", err)
			}
			totalSpace, err := strconv.ParseInt(diskDetail[3], 10, 64)
			if err == nil {
				stat.Total = uint64(totalSpace) / 1073741824
			} else {
				logginghelper.LogError("GetDiskDetails() Error occured while Total space conversion: ", err)
			}

			stat.Used = uint64(totalSpace-freeSpace) / 1073741824
			if stat.Used > 0 && stat.Total > 0 {
				stat.UsedPercent = float64(stat.Used) / float64(stat.Total) * 100
			}
			diskStats = append(diskStats, stat)
		}
	}

	return diskStats, nil
}

func SplitString(driveDetails string) []string {
	stringArray := strings.Split(driveDetails, " ")
	diskDetailArray := []string{}
	for _, val := range stringArray {
		if strings.TrimSpace(val) != "" {
			diskDetailArray = append(diskDetailArray, val)
		}
	}
	return diskDetailArray
}

//AnalyseMemory analyse memory availability on system
func AnalyseMemory() (mem.VirtualMemoryStat, error) {
	virtualMemory, err := mem.VirtualMemory()
	if err != nil {
		logginghelper.LogError("error occured while analysing memory : ", err)
		return mem.VirtualMemoryStat{}, err
	}
	return *virtualMemory, nil
}

// Percent calculates the percentage of cpu used either per CPU or combined.
// If an interval of 0 is given it will compare the current cpu times against the last call.
// Returns one value per cpu, or a single value if percpu is set to false.
func GetCPUUtilizationPercent(interval time.Duration, percpu bool) ([]float64, error) {
	return cpu.Percent(interval, percpu)
}

// GetCPUPhysicalID returns CPU Physical ID or empty string
func GetCPUPhysicalID() (string, error) {
	cpuPhysicalID := ""
	os := ""
	osInfo, err := host.Info()
	if err != nil {
		logginghelper.LogError("error occured while getting OS details", err)
		// return cpuPhysicalID, err
		os = runtime.GOOS
	} else {
		os = osInfo.OS
	}
	switch os {
	case "windows":
		cmd := exec.Command("wmic", "CPU", "get", "ProcessorId")
		// cmd.Stdin = strings.NewReader("CPU get ProcessorId")

		var out bytes.Buffer
		cmd.Stdout = &out
		commandError := cmd.Run()
		if commandError != nil {
			logginghelper.LogError("error occured while running wmic command", commandError)
			return cpuPhysicalID, commandError
		}

		re := regexp.MustCompile("[^\\s(ProcessorId)\\s\r\n]+[0-9 A-Z a-z]+")
		cpuDetailsList := re.FindAllString(out.String(), -1)

		if cpuDetailsList == nil || len(cpuDetailsList) <= 0 {
			logginghelper.LogError("cpu detailslist has no elements to show")
			return cpuPhysicalID, errors.New("cpu detailslist has no elements to show")
		}
		cpuPhysicalID = cpuDetailsList[0]
	case "linux", "darwin":
		// get cpu physical id for linux, darwin OS
		cpuStats, err := cpu.Info()
		if err != nil {
			logginghelper.LogDebug("OUT : GetCPUPhysicalId")
			return cpuPhysicalID, errors.New("error while getting CPU info")
		}
		if len(cpuStats) > 0 {
			logginghelper.LogDebug("OUT : GetCPUPhysicalId")
			cpuPhysicalID = cpuStats[0].PhysicalID
		}
	}
	return strings.TrimSpace(cpuPhysicalID), nil
}
